#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include <chplot.h>
#include <embedch.h>

#define BUTTONID0 1
#define BUTTONID1 2
#define EDITID0	3
#define EDITID1	4
#define EDITID2	5
#define EDITID3	6

LRESULT CALLBACK WindowFunc(HWND, UINT, WPARAM, LPARAM);

char szWinName[] = "MyWin"; // name of window class 

HBITMAP hbit;    // handle of bitmap 
HINSTANCE hInst; // handle to this instance 

ChInterp_t interp; // an instance of Embedded Ch

int createPlot(char *expr, char *Xmin, char *Xmax, char *Points, double plotsizex, double plotsizey) { 
  double xmin, xmax, x, y, *x1, *y1;
  int points, i;

  char *title=expr ,                  // Define labels.
       *xlabel="X",
       *ylabel="Y";
  class CPlot plot;

  xmin = atof(Xmin);
  xmax = atof(Xmax);
  points = atoi(Points);
  x1 = (double *)malloc(points*sizeof(double));
  if(x1 == NULL)
      printf("Error: malloc() failed\n");
  y1 = (double *)malloc(points*sizeof(double));
  if(y1 == NULL)
      printf("Error: malloc() failed\n");
  for (i=0; i<points; i++)
  {
    x = xmin + i*(xmax-xmin)/(points-1);
    /* y = streval(expr); this for Ch only */
    Ch_SetVar(interp, "x", CH_DOUBLETYPE, x); /* set x in Ch space */
    Ch_ExprCalc(interp, expr, CH_DOUBLETYPE, &y); /* calculate expr in Ch space */
    x1[i] = x;
    y1[i] = y;
  }
 
  plot.data2DCurve(x1, y1, points);
  plot.title(title);
  plot.label(PLOT_AXIS_X, xlabel);
  plot.label(PLOT_AXIS_Y, ylabel);
  /* create a png file */
  plot.outputType(PLOT_OUTPUTTYPE_FILE, "png", "test.png");
  plot.size (plotsizex, plotsizey);
  plot.plotting();
  /* change from png to bmp */
  remove("test.bmp");
  /* The command below will create a flashing console in 
     a compile binary version.
     To avoid this console, use ImageMagic C lib 
     from http://www.imagemagick.org/ChMagick/
     to convert png to bmp */
  system("png2bmp  -E -Q -O test.bmp test.png");
  return 0;
}

int WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst, 
                   LPSTR lpszArgs, int nWinMode)
{
  HWND hwnd;
  MSG msg;
  WNDCLASSEX wcl;

  // Define a window class. 
  wcl.cbSize = sizeof(WNDCLASSEX); 

  wcl.hInstance = hThisInst;     // handle to this instance 
  wcl.lpszClassName = szWinName; // window class name 
  wcl.lpfnWndProc = WindowFunc;  // window function 
  wcl.style = 0;                 // default style 

  wcl.hIcon = LoadIcon(NULL, IDI_APPLICATION); // large icon 
  wcl.hIconSm = NULL; // use small version of large icon
  wcl.hCursor = LoadCursor(NULL, IDC_ARROW);  // cursor style

  wcl.lpszMenuName = NULL; // no class menu 
  wcl.cbClsExtra = 0;      // no extra memory needed
  wcl.cbWndExtra = 0; 

  // Make the window background white.
  wcl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH); 

  // Register the window class. 
  if(!RegisterClassEx(&wcl)) return 0;

  hInst = hThisInst; // save the current instance handle 

  /* Now that a window class has been registered, a window
     can be created. */
  hwnd = CreateWindow(
    szWinName, // name of window class 
    "2D Plot", // title 
    WS_OVERLAPPEDWINDOW, // window style - normal 
    CW_USEDEFAULT, // X coordinate - let Windows decide 
    CW_USEDEFAULT, // Y coordinate - let Windows decide 
    CW_USEDEFAULT, // width - let Windows decide 
    CW_USEDEFAULT, // height - let Windows decide
    NULL,          // no parent window 
    NULL,          // no menu
    hThisInst,     // instance handle
    NULL           // no additional arguments 
  );

  // Display the window. 
  ShowWindow(hwnd, nWinMode);
  UpdateWindow(hwnd);

  Ch_Initialize(&interp, NULL); /* initialize Embedded Ch */
  Ch_AppendRunScript(interp,"double x;"); /* declare x in the Ch space */

  // Create the message loop. 
  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg); // translate keyboard messages 
    DispatchMessage(&msg);  // return control to Windows 2000
  }

  Ch_End(interp);         // release Embedded Ch */
  return msg.wParam;
}

/* This function is called by Windows 2000 and is passed 
   messages from the message queue.
*/
LRESULT CALLBACK WindowFunc(HWND hwnd, UINT message,
                         WPARAM wParam, LPARAM lParam)
{
  HDC hdc, memdc;
  PAINTSTRUCT ps;
  static HWND  hwndButton[2] ;
  static HWND  hwndEdit[4] ;
  static HRGN  hrgn;
  HBRUSH	hbrushb;
  HPEN	hPen, hOldPen;
  RECT	rect;
  int   i, iLength;
  char	szbuffer[128];

  static char Editstring0[128] = "3*sin(2*x)";
  static char Editstring1[64] = "0";
  static char Editstring2[64] = "6";
  static char Editstring3[64] = "50";
  static double xsize = 0.75, ysize = 0.75, xclient = 1, yclient = 1;
  static int iCount = 0;

  switch(message) {
    case WM_CREATE: 
        GetClientRect (hwnd, &rect);
		xclient = rect.right - rect.left;
		yclient = rect.bottom - rect.top;

        hwndButton[0] = CreateWindow ("button", "Plot",
                              WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
                              520, 20 , 60, 25, hwnd, (HMENU) BUTTONID0, hInst, NULL);
        hwndButton[1] = CreateWindow ("button", "Reset",
                              WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
                              600, 20 , 80, 25, hwnd, (HMENU) BUTTONID1, hInst, NULL);
        hwndEdit[0] = CreateWindow ("edit", NULL, WS_CHILD | WS_VISIBLE | 
                              WS_BORDER | ES_LEFT | ES_MULTILINE |
                              ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
							  130, 20, 330, 25,
							  hwnd, (HMENU) EDITID0, hInst, NULL) ;
        hwndEdit[1] = CreateWindow ("edit", NULL, WS_CHILD | WS_VISIBLE | 
                              WS_BORDER | ES_LEFT | ES_MULTILINE |
                              ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
							  75, 70, 80, 25,
							  hwnd, (HMENU) EDITID1, hInst, NULL) ;
        hwndEdit[2] = CreateWindow ("edit", NULL, WS_CHILD | WS_VISIBLE | 
                              WS_BORDER | ES_LEFT | ES_MULTILINE |
                              ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
							  270, 70, 80, 25,
							  hwnd, (HMENU) EDITID2, hInst, NULL) ;
        hwndEdit[3] = CreateWindow ("edit", NULL, WS_CHILD | WS_VISIBLE | 
                              WS_BORDER | ES_LEFT | ES_MULTILINE |
                              ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
							  565, 70, 100, 25,
							  hwnd, (HMENU) EDITID3, hInst, NULL) ;

      SetWindowText (hwndEdit[0], Editstring0) ;
	  SetWindowText (hwndEdit[1], Editstring1) ;
	  SetWindowText (hwndEdit[2], Editstring2) ;
	  SetWindowText (hwndEdit[3], Editstring3) ;
      break;

    case WM_PAINT:
      hdc = BeginPaint(hwnd, &ps); // get device context 

	  hbrushb = CreateSolidBrush(RGB(200, 200, 200));
	  GetClientRect (hwnd, &rect);
	  FillRect(hdc, &rect, hbrushb);
	  SelectObject (hdc,GetStockObject(WHITE_BRUSH));
	  Rectangle (hdc, rect.left+5, rect.top+120, rect.right-5, rect.bottom-5); //Plot Rect
	  hPen = CreatePen(PS_SOLID, 2, RGB(80, 80, 80));
	  hOldPen = (HPEN) SelectObject(hdc, hPen);
//	  MoveToEx (hdc, 130, 44, NULL);
//	  LineTo (hdc, 130, 20);
//	  MoveToEx (hdc, 130, 22, NULL);
//    LineTo (hdc, 459, 22);
	  MoveToEx (hdc, rect.left+5, rect.bottom-6, NULL);
	  LineTo (hdc, rect.left+5, rect.top+120);
	  MoveToEx (hdc, rect.left+5, rect.top+121, NULL);
      LineTo (hdc, rect.right-6, rect.top+121);
	  SelectObject(hdc, hOldPen);
	  SetBkMode (hdc, TRANSPARENT);	//Ignore the background color
	  iLength=sprintf(szbuffer,"Function :  y =");
	  TextOut(hdc,10,20,szbuffer,iLength);
	  iLength=sprintf(szbuffer,"X-min :");
	  TextOut(hdc,10,70,szbuffer,iLength);
	  iLength=sprintf(szbuffer,"X-max :");
	  TextOut(hdc,200,70,szbuffer,iLength);
	  iLength=sprintf(szbuffer,"Number of points :");
	  TextOut(hdc,400,70,szbuffer,iLength);

	  memdc = CreateCompatibleDC(hdc); // create compatible DC 
      SelectObject(memdc, hbit); // select bitmap 
      BitBlt(hdc, 120, 140, rect.right, rect.bottom,
             memdc, 0, 0, SRCCOPY); // display image
	  
      EndPaint(hwnd, &ps); // release DC 
      DeleteDC(memdc); // free the memory context 
	  DeleteObject(hbrushb);
	  DeleteObject(hPen);

      break;

    case WM_COMMAND :
      switch (LOWORD (wParam))
      {
	    case BUTTONID0 :
          iLength = GetWindowTextLength (hwndEdit[0]) ;
          GetWindowText (hwndEdit[0], Editstring0, iLength + 1) ;
          iLength = GetWindowTextLength (hwndEdit[1]) ;
          GetWindowText (hwndEdit[1], Editstring1, iLength + 1) ;
          iLength = GetWindowTextLength (hwndEdit[2]) ;
          GetWindowText (hwndEdit[2], Editstring2, iLength + 1) ;
          iLength = GetWindowTextLength (hwndEdit[3]) ;
          GetWindowText (hwndEdit[3], Editstring3, iLength + 1) ;
          createPlot(Editstring0, Editstring1, Editstring2, Editstring3, xsize, ysize);
          hbit = (HBITMAP)LoadImage(hInst, "test.bmp", IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	      InvalidateRect (hwnd, NULL, TRUE) ;
		  return 0;
        case BUTTONID1 :
		  sprintf(Editstring0,"3*sin(2*x)");
		  SetWindowText (hwndEdit[0], Editstring0) ;
		  sprintf(Editstring1,"0");
		  SetWindowText (hwndEdit[1], Editstring1) ;
		  sprintf(Editstring2,"6");
		  SetWindowText (hwndEdit[2], Editstring2) ;
		  sprintf(Editstring3,"50");
		  SetWindowText (hwndEdit[3], Editstring3) ;
		  return 0;
	  }
      break;

	case WM_SIZE :
	  if( iCount != 0 )
	  {
	    GetClientRect (hwnd, &rect);
	    xsize = 0.75*((rect.right-rect.left)/xclient);
	    ysize = 0.75*((rect.bottom-rect.top)/yclient);
        createPlot(Editstring0, Editstring1, Editstring2, Editstring3, xsize, ysize);
        hbit = (HBITMAP)LoadImage(hInst, "test.bmp", IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	    InvalidateRect (hwnd, NULL, TRUE) ;
	  }
	  iCount ++;
	  break;

    case WM_DESTROY: // terminate the program 
      DeleteObject(hbit); // remove the bitmap
      PostQuitMessage(0);
      break;

    default:
      /* Let Windows 2000 process any messages not specified in
         the preceding switch statement. */
      return DefWindowProc(hwnd, message, wParam, lParam);
  }

  return 0;
}


